import {Component, EventEmitter, Input, OnInit, Output} from '@angular/core';
import {
  MatTableDataSource
} from "@angular/material/table";
import {LeafGamesystemCalculator} from "../../product-gamesystem-editor/LeafGamesystemCalculator";
import {animate, state, style, transition, trigger} from "@angular/animations";
import {ProductGamesystem} from "../../../../project/game-model/gamesystems/ProductGamesystem";
import {SimpleGamesystem} from "../../../../project/game-model/gamesystems/SimpleGamesystem";
import {ProductTransition} from "../../../../project/game-model/gamesystems/transitions/ProductTransition";
import {ProductState} from "../../../../project/game-model/gamesystems/states/ProductState";
import {TemplateElement} from "../../../../project/game-model/templates/TemplateElement";
import {ProductTemplateSystem} from "../../../../project/game-model/templates/productGamesystem/ProductTemplateSystem";
class DisplayedColumnName {
  displayedName: string
  internalName: string


  constructor(displayedName: string, internalName: string) {
    this.displayedName = displayedName;
    this.internalName = internalName;
  }
}
@Component({
  selector: 'app-product-transition-editor',
  templateUrl: './product-transition-editor.component.html',
  styleUrl: './product-transition-editor.component.scss',
  animations: [
    trigger('detailExpand', [
      state('collapsed,void', style({height: '0px', minHeight: '0'})),
      state('expanded', style({height: '*'})),
      transition('expanded <=> collapsed', animate('225ms cubic-bezier(0.4, 0.0, 0.2, 1)')),
    ]),
  ],
})
export class ProductTransitionEditorComponent implements OnInit{

  @Input() gamesystem: ProductGamesystem | undefined
  @Output() onOpenGamesystem = new EventEmitter<SimpleGamesystem>();
  @Input() templateElement: TemplateElement | undefined

  dataSource = new MatTableDataSource<ProductTransition>();
  displayedColumns: DisplayedColumnName[] = [];
  columns: string[] = [];
  columnsToDisplayWithExpand = [...this.columns, 'expand'];
  expandedElement: ProductTransition | null = null
  numberLeafSystems: number = -1;

  ngOnInit() {
    if(this.gamesystem != undefined) {
      const leafGamesystems: SimpleGamesystem[] = LeafGamesystemCalculator.calcLeafGeamesystems(this.gamesystem);
      this.displayedColumns = leafGamesystems.map(leafGamesystem => new DisplayedColumnName(leafGamesystem.componentName, leafGamesystem.componentName + "-start"));
      this.displayedColumns = this.displayedColumns.concat( leafGamesystems.map(leafGamesystem => new DisplayedColumnName(leafGamesystem.componentName, leafGamesystem.componentName + "-end")));
      this.renameDuplicateColumnNames(this.displayedColumns)


      this.numberLeafSystems = leafGamesystems.length;
      this.columns = this.displayedColumns.map(column => column.internalName)
      this.columnsToDisplayWithExpand = [...this.columns, 'expand']

      if(this.templateElement == undefined) {
        this.dataSource.data = this.gamesystem.transitions;
      } else if(this.gamesystem instanceof ProductTemplateSystem){
        this.dataSource.data = this.gamesystem!.transitionMap.get(this.templateElement)!
      }

      this.dataSource.filterPredicate = (data: ProductTransition, filter: string) => {
        const leaf_starting_states = LeafGamesystemCalculator.calcLeafStates(data.startingState);
        const leaf_ending_states = LeafGamesystemCalculator.calcLeafStates(data.endingState);

        const combined_leaf_states = leaf_starting_states.concat(leaf_ending_states);

        return combined_leaf_states.some((state) => state.stateLabel.toLowerCase().includes(filter))
      }
    }
  }

  private renameDuplicateColumnNames(columnNames: DisplayedColumnName[]) {
    for(let i=0; i<columnNames.length; i++) {
      for(let j=i; j<columnNames.length; j++) {
        if(i === j) {
          continue;
        }

        if(columnNames[i].internalName === columnNames[j].internalName) {
          columnNames[j].internalName = columnNames[j].internalName + "-" + i
        }
      }
    }
  }

  getLeafStateByIndex(transition: ProductTransition, leafIndex: number) {
    let state: ProductState;
    let index = leafIndex;
    if(leafIndex < this.numberLeafSystems) {
      state = transition.startingState
    } else {
      state = transition.endingState;
      index = leafIndex - this.numberLeafSystems;
    }

    const leafStates = LeafGamesystemCalculator.calcLeafStates(state);
    return leafStates[index];
  }

  openGamesystemEditor(leafIndex: number) {
    const leafGamesystems = LeafGamesystemCalculator.calcLeafGeamesystems(this.gamesystem!);
    if(leafIndex < this.numberLeafSystems) {
      this.onOpenGamesystem.emit(leafGamesystems[leafIndex])
    } else {
      this.onOpenGamesystem.emit(leafGamesystems[leafIndex - this.numberLeafSystems])
    }
  }

  applyFilter(event: KeyboardEvent) {
    const filterValue = (event.target as HTMLInputElement).value;
    this.dataSource.filter = filterValue.trim().toLowerCase();
  }
}
